import pytest
from utils import run_query_using_timbr_dialect, get_connection_uri_using_timbr_dialect

def test_run_query(test_config):
  uri = get_connection_uri_using_timbr_dialect(
    hostname=test_config['hostname'],
    port=test_config['port'],
    protocol=test_config['protocol'],
    ontology=test_config['ontology'],
    username=test_config['username'],
    password=test_config['password']
  )
  results_obj = run_query_using_timbr_dialect(uri, "SHOW CONCEPTS", connect_args=test_config['connect_args'])
  results_data = results_obj["results"]
  results_headers = results_obj["headers"]

  assert results_obj is not None, "Query did not return any results"
  assert len(results_data) > 0, "Query returned no rows"
  assert len(results_headers) > 0, "Query returned no columns"
  assert all(len(row) == len(results_headers) for row in results_data), "Row length does not match header length"


from sqlalchemy.engine import create_engine

if __name__ == '__main__':
  # Declare the connection variables

  # General example
  hostname = '<TIMBR_IP/HOST>'
  port = '<TIMBR_PORT>'
  ontology = '<ONTOLOGY_NAME>'
  protocol = '<http/https>'
  username = '<TIMBR_USER/token>'
  password = '<TIMBR_PASSWORD/TOKEN_VALUE>'

  # hostname - The IP / Hostname of the Timbr server (not necessarily the hostname of the Timbr platform).
  # port - The port to connect to in the Timbr server. Timbr's default port with enabled_ssl is 443 without SSL is 11000.
  # ontology = The name of the ontology (knowledge graph) to connect.
  # protocol - Connection protocol can be 'http' or 'https'.
  # username - Use 'token' as the username when connecting using a Timbr token, otherwise use the user name.
  # password - If using a token as a username then the pass is the token value, otherwise its the user's password.

  # HTTP example
  hostname = 'mytimbrenv.com'
  port = '11000'
  ontology = 'my_ontology'
  protocol = 'http'
  username = 'timbr'
  password = 'StrongPassword'

  # HTTPS example
  hostname = 'mytimbrenv.com'
  port = '443'
  ontology = 'my_ontology'
  protocol = 'https'
  username = 'timbr'
  password = 'StrongPassword'

  # Create new sqlalchemy connection
  engine = create_engine(f"timbr+{protocol}://{username}@{ontology}:{password}@{hostname}:{port}")

  # Connect to the created engine
  conn = engine.connect()

  # Execute a query
  query = "SHOW CONCEPTS"
  res_obj = conn.execute(query)
  results_headers = [(desc[0], desc[1]) for desc in res_obj.cursor.description]

  # Optional Performance Tuning:
  # By default, Timbr fetches results in batches of 10,000 rows at a time.
  # You can increase this batch size for better performance with large datasets.
  # Note: The maximum batch size is controlled by your Timbr server's 
  # "TIMBR_RESULTSET_MAX_FETCH_SIZE" configuration setting.
  res_obj.cursor._arraysize = 20000

  results = res_obj.fetchall()

  # Print the columns name
  for name, col_type in results_headers:
    print(f"{name} - {col_type}")
  # Print the results
  for result in results:
    print(result)


import pytest
from utils import get_connection_uri_using_hive_dialect, run_query_using_hive_dialect

def create_engine_and_run_query(config, is_async=False):
  """
  Creates a SQLAlchemy engine and runs a query using the Hive dialect.
  """
  uri = get_connection_uri_using_hive_dialect(
    hostname=config['hostname'],
    port=config['port'],
    protocol=config['protocol'],
    ontology=config['ontology'],
    username=config['username'],
    password=config['password']
  )
  return run_query_using_hive_dialect(
    uri,
    "SHOW CONCEPTS",
    config['connect_args'],
    is_async,
  )

def test_run_sync_query(test_config):
  results_obj = create_engine_and_run_query(test_config, is_async=False)
  results_data = results_obj["results"]
  results_headers = results_obj["headers"]

  assert results_obj is not None, "Query did not return any results"
  assert len(results_data) > 0, "Query returned no rows"
  assert len(results_headers) > 0, "Query returned no columns"
  assert all(len(row) == len(results_headers) for row in results_data), "Row length does not match header length"

def test_run_async_query(test_config):
  results_obj = create_engine_and_run_query(test_config, is_async=True)
  results_data = results_obj["results"]
  results_headers = results_obj["headers"]

  assert results_obj is not None, "Query did not return any results"
  assert len(results_data) > 0, "Query returned no rows"
  assert len(results_headers) > 0, "Query returned no columns"
  assert all(len(row) == len(results_headers) for row in results_data), "Row length does not match header length"


"""Timbr Python SQLAlchemy connector."""

__version__ = "2.1.0"

try:
  from thrift.transport import THttpClient
  def timbr_flush(self):
    if self.__http_response is not None:
        self.__http_response.close()
        self.__http_response = None

    if not self.isOpen():
        self.open()

    # Pull data out of buffer
    data = self.__wbuf.getvalue()
    self.__wbuf = BytesIO()

    # HTTP request
    if self.using_proxy() and self.scheme == "http":
        # need full URL of real host for HTTP proxy here (HTTPS uses CONNECT tunnel)
        self.__http.putrequest('POST', "http://%s:%s%s" %
                               (self.realhost, self.realport, self.path))
    else:
        self.__http.putrequest('POST', self.path)

    # Write headers
    self.__http.putheader('Content-Type', 'application/x-thrift')
    self.__http.putheader('Content-Length', str(len(data)))
    if self.using_proxy() and self.scheme == "http" and self.proxy_auth is not None:
        self.__http.putheader("Proxy-Authorization", self.proxy_auth)

    if not self.__custom_headers or 'User-Agent' not in self.__custom_headers:
        user_agent = 'Python/THttpClient'
        script = os.path.basename(sys.argv[0])
        if script:
            user_agent = '%s (%s)' % (user_agent, urllib.parse.quote(script))
        self.__http.putheader('User-Agent', user_agent)

    if self.__custom_headers:
        for key, val in six.iteritems(self.__custom_headers):
            self.__http.putheader(key, val)

    # Saves the cookie sent by the server in the previous response.
    # HTTPConnection.putheader can only be called after a request has been
    # started, and before it's been sent.
    if self.headers and 'Set-Cookie' in self.headers:
        self.__http.putheader('Cookie', self.headers['Set-Cookie'])

    self.__http.endheaders()

    # Write payload
    self.__http.send(data)

    # Get reply to flush the request
    self.__http_response = self.__http.getresponse()
    self.code = self.__http_response.status
    self.message = self.__http_response.reason
    self.headers = self.__http_response.msg

  THttpClient.flush = timbr_flush
except:
    pass



import os
import json
import pytest
from dotenv import load_dotenv
from sqlalchemy.engine import create_engine
from TCLIService.ttypes import TOperationState

# Load environment variables from .env
load_dotenv(override=True)

def test_timbr_dialect():
  # Declare the connection variables
  hostname = os.getenv("HOSTNAME")
  port = os.getenv("PORT")
  protocol = os.getenv("PROTOCOL")
  ontology = os.getenv("ONTOLOGY")
  username = os.getenv("USERNAME")
  password = os.getenv("PASSWORD") 
  connect_args_data = os.getenv("CONNECT_ARGS")
  connect_args = json.loads(connect_args_data)

  assert hostname is not None, "Missing HOSTNAME"
  assert port is not None, "Missing PORT"
  assert protocol is not None, "Missing PROTOCOL"
  assert ontology is not None, "Missing ONTOLOGY"
  assert username is not None, "Missing USERNAME"
  assert password is not None, "Missing PASSWORD"

  # Create new sqlalchemy connection
  engine = create_engine(f"timbr+{protocol}://{username}@{ontology}:{password}@{hostname}:{port}")

  # Connect to the created engine
  conn = engine.connect()

  # Execute a query
  query = "SHOW CONCEPTS"
  res_obj = conn.execute(query)

  results_headers = res_obj.keys()
  results = res_obj.fetchall()

  # Display the results of the execution formatted as a table
  # Print the columns name
  print(f"index | {' | '.join(results_headers)}")
  # Print a separator line
  print("-" * ((len(results_headers)+1) * 10))
  # Print the results
  for res_index, result in enumerate(results, start=1):
    print(f"{res_index} | {' | '.join(map(str, result))}")


























if __name__ == '__main__':

  # Declare the connection variables
  hostname = os.getenv("HOSTNAME")
  port = os.getenv("PORT")
  protocol = os.getenv("PROTOCOL")
  ontology = os.getenv("ONTOLOGY")
  username = os.getenv("USERNAME")
  password = os.getenv("PASSWORD")
  password = os.getenv("PASSWORD")
  connect_args_data = os.getenv("CONNECT_ARGS")
  connect_args = json.load(dict(connect_args_data))



  # connection_args_file_path = os.getenv("CONNECT_ARGS_FILE")
  # if not connection_args_file_path:
  #   raise RuntimeError("Missing CONNECT_ARGS_FILE in .env")

  # print(f"Loading connection args from: {connection_args_file_path}")
  # try:
  #   with open(connection_args_file_path) as f:
  #     connect_args = dict(json.load(f))
  # except Exception as e:
  #   raise RuntimeError(f"Failed to load connect args JSON: {e}")
  # print(f"Loaded connect args: {connect_args}")
  assert hostname is not None, "Missing HOSTNAME"
  assert port is not None, "Missing PORT"
  assert protocol is not None, "Missing PROTOCOL"
  assert ontology is not None, "Missing ONTOLOGY"
  assert username is not None, "Missing USERNAME"
  assert password is not None, "Missing PASSWORD"

  # HTTPS example

  try:
    # example file
    print("RUNNING REGULAR QUERY: \n")
    # Create new sqlalchemy connection
    engine = create_engine(f"timbr+{protocol}://{username}@{ontology}:{password}@{hostname}:{port}")

    # Connect to the created engine
    conn = engine.connect()

    # Execute a query
    query = "SHOW CONCEPTS"
    res_obj = conn.execute(query)

    results_headers = res_obj.keys()
    results = res_obj.fetchall()

    # Display the results of the execution formatted as a table
    # Print the columns name
    print(f"index | {' | '.join(results_headers)}")
    # Print a separator line
    print("-" * ((len(results_headers)+1) * 10))
    # Print the results
    for res_index, result in enumerate(results, start=1):
      print(f"{res_index} | {' | '.join(map(str, result))}")

    print("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n")
    print("RUNNING ASYNC QUERY: \n")



    # async pyhive

    # Create new sqlalchemy connection
    engine = create_engine(f"hive+{protocol}://{username}@{ontology}:{password}@{hostname}:{port}", connect_args={'configuration': {'set:hiveconf:hiveMetadata': 'true'}})

    # Connect to the created engine
    conn = engine.connect()
    dbapi_conn = engine.raw_connection()
    cursor = dbapi_conn.cursor()

    # Execute a query
    query = "SHOW CONCEPTS"
    cursor.execute(query)

    # Check the status of this execution
    status = cursor.poll().operationState
    while status in (TOperationState.INITIALIZED_STATE, TOperationState.RUNNING_STATE):
      status = cursor.poll().operationState

    # Get the results of the execution
    results_headers = [(desc[0], desc[1]) for desc in cursor.description]
    results = cursor.fetchall()

    # Display the results of the execution
    # Print the columns name
    for name, col_type in results_headers:
      print(f"{name} - {col_type}")
    # Print the results
    for result in results:
      print(result)

    print("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n")
    print("RUNNING SYNC QUERY: \n")



    # sync pyhive

    # Create new sqlalchemy connection
    engine = create_engine(f"timbr+{protocol}://{username}@{ontology}:{password}@{hostname}:{port}", connect_args=connect_args)

    # Connect to the created engine
    conn = engine.connect()

    # Use the connection to execute a query
    query = "SHOW CONCEPTS"
    # query = "select sleep(20)"
    # query = "SHOW SCHEMAS"
    res_obj = conn.execute(query)
    results_headers = [(desc[0], desc[1]) for desc in res_obj.cursor.description]
    results = res_obj.fetchall()

    # Print the columns name
    for name, col_type in results_headers:
      print(f"{name} - {col_type}")
    # Print the results
    for result in results:
      print(result)


    print("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n")
    
  except Exception as e:
    print(e)